CONFIG_TOPIC = "homeassistant/{sensor_type}/grott/{device}_{attribut}/config"
STATE_TOPIC = "homeassistant/grott/{device}/state"

MQTT_HOST_CONF_KEY = "ha_mqtt_host"
MQTT_PORT_CONF_KEY = "ha_mqtt_port"
MQTT_USERNAME_CONF_KEY = "ha_mqtt_user"
MQTT_PASSWORD_CONF_KEY = "ha_mqtt_password"
MQTT_RETAIN_CONF_KEY = "ha_mqtt_retain"
